#! /usr/bin/env python3

# Script for comparing the md5sum of a posix file with the md5sums of a multi chunk swift object
#
# swhashcomp dirkpetersen / Feb 2015 
#

import swiftclient, sys, os, argparse, functools, hashlib, json

class KeyboardInterruptError(Exception): pass

def main():

    c=create_sw_conn()
    md5all = hashlib.md5()

    print ("    comparing swift object %s/%s with %s..." % (args.container, args.obj, args.locfile))

    #headers, objects = c.get_container(args.container,prefix=args.prefix,full_listing=True)
    headers = c.head_object(args.container, args.obj)

    if 'x-static-large-object' in headers:
        #print(headers['x-static-large-object'])
        headers, body = c.get_object(args.container, args.obj, query_string='multipart-manifest=get')
        if not os.path.isfile(args.locfile):
            if 'md5sum' in headers:
                if args.locfile.strip() == headers['md5sum']:
                    print('    md5sum:%s' % headers['md5sum'])
                    is_valid = True
                else:
                    is_valid = False
            else:
                is_valid = check_segments(body,args.locfile.strip(),c)
        else:
            if os.path.splitext(args.locfile)[1].lower() in ['.md5', '.sha1']:
                with open(args.locfile, 'r') as f:
                    myhash = f.read().split(None,1)[0]
                    print('md5sum from md5 file: %s' % myhash)
                    is_valid = check_segments(body,myhash,c)
            else:
                with open(args.locfile, 'rb') as f:
                    is_valid = check_manifest(body, f, md5all)
    else:
        is_valid=False
        if os.path.isfile(args.locfile):
            if os.path.splitext(args.locfile)[1].lower() in ['.md5', '.sha1']:
                with open(args.locfile, 'r') as f:
                    myhash = f.read().split(None,1)[0]	
                print('md5sum from md5 file: %s' % myhash)
            else:
                with open(args.locfile, 'rb') as f:
                    hasher = hashlib.md5(f.read()) # needed for compatiblity between python3 and python2
                    myhash = hasher.hexdigest()
                print('md5sum of local file: %s' % myhash)
            if myhash == headers['etag']:
                print('    md5sum:%s' % headers['etag'])
                is_valid = True
        else:
            if args.locfile.strip() == headers['etag']:
                print('    md5sum:%s' % headers['etag'])
                is_valid = True

    if is_valid:
        print ("object %s/%s and '%s' are identical!" % (args.container, args.obj, args.locfile))
        return 0
    else:
        print ("*** WARNING ***: object %s/%s and '%s' are different!" % (args.container, args.obj, args.locfile))
        return 1

def check_manifest(manifest, body, md5all):
    """
    check if a body is the same object described by the manifest

    :param manifest: the raw body of the manifest from swift
    :param body: a file like object to check against the manfiest
    """
    manifest = json.loads(manifest.decode())
    for segment in manifest:
        print ("    testing chunk %s" % segment['name'])
        chunk = body.read(segment['bytes'])
        hasher = hashlib.md5(chunk)
        md5all.update(chunk)
        if hasher.hexdigest() != segment['hash']:
            print ('    %s != %s' % (hasher.hexdigest(), segment['hash']))            
            return False
    print("    md5sum:%s" % md5all.hexdigest())
    return True

def check_segments(manifest,md5sum,c):
    manifest = json.loads(manifest.decode())
    digest = hashlib.md5()
    for segment in manifest:
        print ("    please wait ... testing chunk %s" % segment['name'])
        segment_container, segment_obj = parseSwiftUrl(segment['name'])
        attributes, content = c.get_object(segment_container, segment_obj)
        digest.update(content)
    if digest.hexdigest() != md5sum:
        print ('    %s != %s' % (digest.hexdigest(), md5sum))
        return False
    return True

def create_sw_conn():
    if args.authtoken and args.storageurl:
        return swiftclient.Connection(preauthtoken=args.authtoken, preauthurl=args.storageurl)
    else:
        authtoken=os.environ.get("OS_AUTH_TOKEN")
        storageurl=os.environ.get("OS_STORAGE_URL")
        if authtoken and storageurl:
            return swiftclient.Connection(preauthtoken=authtoken, preauthurl=storageurl)
        else:
            swift_auth=os.environ.get("ST_AUTH")
            swift_user=os.environ.get("ST_USER")
            swift_key=os.environ.get("ST_KEY")
            if swift_auth and swift_user and swift_key:
                return swiftclient.Connection(authurl=swift_auth,user=swift_user,key=swift_key)
                
def parseSwiftUrl(path):
    path = path.lstrip('/')
    components = path.split('/');
    container = components[0];
    obj = '/'.join(components[1:])
    return container, obj

def parse_arguments():
    """
    Gather command-line arguments.
    """

    parser = argparse.ArgumentParser(prog='swhashcomp',
        description='compare the md5sum of a local file or hash with the hash ' + \
        'of a swift object folder after a data migration ' + \
        '')
    parser.add_argument( '--locfile', '-f', dest='locfile',
        action='store',
        help='a local or networked file to compare',
        default='' )        
    parser.add_argument( '--container', '-c', dest='container',
        action='store',
        help='a container in the swift object store',
        default='' )
    parser.add_argument( '--obj', '-o', dest='obj',
        action='store',
        help='an object in a swift container',
        default=None)
    parser.add_argument( '--authtoken', '-a', dest='authtoken',
        action='store',
        help='a swift authentication token (required when storage-url is used)',
        default=None)
    parser.add_argument( '--storage-url', '-s', dest='storageurl',
        action='store',
        help='a swift storage url (required when authtoken is used)',
        default=None)
    args = parser.parse_args()
    if not args.locfile:
        parser.error('required option --locfile not given !')
    if not args.container:
        parser.error('required option --container not given !')
    if not args.obj:
        parser.error('required option --obj not given !')
    return args

if __name__ == '__main__':
    # Parse command-line arguments
    args = parse_arguments()
    sys.exit(main())




#! /usr/bin/env python3

# Script to save or restore all symlinks in a directory structure to a textfile 
# name: .symbolic-links.tree.txt
#
# swsymlinks dirkpetersen / Aug 2015
#

import sys, os, pwd, argparse, subprocess, re, time, datetime, tempfile
try:
    from scandir import walk
except:
    from os import walk

class KeyboardInterruptError(Exception): pass

def main():
    global args

    # Parse command-line arguments
    args = parse_arguments()

    oldcurrdir = os.getcwd()
    os.chdir(args.folder)
    currdir = os.getcwd() 
    curruser = pwd.getpwuid(os.getuid()).pw_name
    tmpdir = tempfile.gettempdir()

    if args.folder.startswith('./'):
        args.folder = args.folder[2:]

    if os.path.abspath(currdir) != os.path.abspath(args.folder):
        print("Current folder %s <> target folder %s, exiting....", (currdir,args.folder))
        return False

    if args.linkdbfolder and args.restore:
        print("You cannot use --linkdbfolder in --restore mode.")
        print("Please put .symbolic-links.tree.txt in the root of --folder")
        return False

    linkdbfolder = currdir
    if args.linkdbfolder:
        linkdbfolder = args.linkdbfolder
                
    fcnt=0
    dcnt=0    
    if args.save:
        print ('\nScanning folder %s to archive symlinks ...' % currdir)
        if args.single:
            # only create one single .symbolic-links.tree.txt at the root
                with open(os.path.join(linkdbfolder,".symbolic-links.tree.txt"),'w') as sfile:
                    dcnt+=1
                    for root, folders, files in mywalk(args.folder):
                        for f in files:
                            try:
                                base=root.replace(currdir+'/','')
                                p=os.path.join(base,f)
                                if os.path.islink(p):
                                    stat=getstat(p)
                                    targ=os.readlink(p)
                                    sfile.write("%s|%s|%s\n" % (p,targ,stat.st_mtime))
                                    fcnt+=1
                                    if args.debug:
                                        sys.stderr.write('SYMLINK:%s\n   TARGET:%s\n' % (p,targ))
                                    
                            except Exception as err:
                                sys.stderr.write(str(err))
                                sys.stderr.write('\n')

        else:
            # create a .symbolic-links.tree.txt in each folder recursively
            for root, folders, files in mywalk(args.folder):
                try:
                    base=root.replace(currdir+'/','')
                    treefile=os.path.join(base,'.symbolic-links.tree.txt')
                    if os.path.exists(treefile):
                        os.remove(treefile)
                    oldroot=''
                except Exception as err:
                    sys.stderr.write(str(err))
                    sys.stderr.write('\n')                    
                for f in files:
                    try:
                        p=os.path.join(base,f)
                        if os.path.islink(p):
                            if oldroot != root:
                                dcnt+=1
                                oldroot=root
                                sys.stderr.write('Symlink Store: %s\n' % (treefile))
                            stat=getstat(p)
                            targ=os.readlink(p)
                            with open(os.path.join(root,'.symbolic-links.tree.txt'),'a') as sfile:
                                sfile.write("%s|%s|%s\n" % (f,targ,stat.st_mtime))
                            fcnt+=1
                            if args.debug:
                                sys.stderr.write('  SYMLINK:%s\n    TARGET:%s\n' % (p,targ))

                    except Exception as err:
                        sys.stderr.write(str(err))
                        sys.stderr.write('\n')
                    
        print("saved %i symlinks into %i .symbolic-links.tree.txt store(s)" % (fcnt, dcnt))

        
    elif args.clean:
        # remove all .symbolic-links.tree.txt files recursively
        cnt=0
        print ('recursively clean .symbolic-links.tree.txt from %s' % currdir)
        for root, folders, files in mywalk(args.folder):            
            if args.debug:
                print ("cleaning folder %s" % root)
            p=os.path.join(root,'.symbolic-links.tree.txt')
            if os.path.exists(p):
                os.remove(p)
                print('deleted %s' % p)                
                cnt+=1
        print('%i .symbolic-links.tree.txt file(s) have been removed.' % cnt)
        
    elif args.restore:
        for root, folders, files in mywalk(args.folder):
            base=root.replace(currdir+'/','')
            treefile=os.path.join(base,'.symbolic-links.tree.txt')
            if os.path.exists(treefile):
                sys.stderr.write('restoring from Symlink Store: %s ...\n ' % (treefile))
                dcnt+=1
                lines=[]
                with open(treefile,'r') as sfile:
                    lines=sfile.readlines()
                for line in lines:
                    try:
                        link,targ,mtime=line.split('|')                    
                        d=os.path.dirname(link)
                        if d:
                            #link was saved with a directory in .symbolic-links.tree.txt
                            if not os.path.exists(d):
                                os.makedirs(d)
                        else:
                            #link was saved as a file only in .symbolic-links.tree.txt
                            link=os.path.join(base,link)
                        if os.path.islink(link) and args.force:
                            os.unlink(link)
                        if not os.path.islink(link):
                            os.symlink(targ,link)
                            stat=getstat(link)
                            #set both symlink atime and mtime to mtime
                            os.utime(link,(float(mtime),float(mtime)),follow_symlinks=False)
                            fcnt+=1
                            if args.debug:
                                sys.stderr.write('SYMLINK:%s\n   TARGET:%s\n' % (link,targ))
                    except Exception as err:
                        sys.stderr.write(str(err))
                        sys.stderr.write('\n')
        print("Restored %i symlinks from %i .symbolic-links.tree.txt store(s)" % (fcnt, dcnt))
    else:
        print("You need to use either --save, --save --single, --clean or --restore as a command option")

    os.chdir(oldcurrdir)
        

def startswithpath(pathlist, pathstr):
    """ checks if at least one of the paths in a list of paths starts with a string """
    for path in pathlist:
        if (os.path.join(pathstr, '')).startswith(path):
            return True
    return False

def getstartpath(pathlist, pathstr):
    """ return the path from pathlist that is the first part of pathstr"""
    for path in pathlist:
        if (os.path.join(pathstr, '')).startswith(path):
            return path
    return ''

                
def getstat(path):
    """ returns the stat information of a file"""
    statinfo=None
    try:
        statinfo=os.lstat(path)
    except (IOError, OSError) as e:   # FileNotFoundError only since python 3.3
        if args.debug:
            sys.stderr.write(str(e))
    except:
        raise
    return statinfo

def setfiletime(path,attr="atime"):
    """ sets the a time of a file to the current time """
    try:
        statinfo=getstat(path)
        if attr=="atime" or attr=="all":
            os.utime(path,(time.time(),statinfo.st_atime))
        if attr=="mtime" or attr=="all":
            os.utime(path,(time.time(),statinfo.st_mtime))        
        return True
    except Exception as err:
        sys.stderr.write(str(err))
        sys.stderr.write('\n')
        return False

def uid2user(uidNumber):
    """ attempts to convert uidNumber to username """
    import pwd
    try:
        return pwd.getpwuid(int(uidNumber)).pw_name
    except Exception as err:
        sys.stderr.write(str(err))
        sys.stderr.write('\n')
        return str(uidNumber)

def list2file(mylist,path):
    """ dumps a list into a text file, one line per item"""
    try:
        with open(path,'w') as f:
            for item in mylist:
                f.write("{}\r\n".format(item))
        return True
    except Exception as err:
        sys.stderr.write(str(err))
        sys.stderr.write('\n')
        return False

def pathlist2file(mylist,path,root):
    """ dumps a list into a text file, one line per item, but removes
         a root folder from all paths. Used for --files-from feature in rsync"""
    try:
        with open(path,'w') as f:
            for item in mylist:
                f.write("{}\r\n".format(item[len(root):]))
        return True
    except Exception as err:
        sys.stderr.write(str(err))
        sys.stderr.write('\n')
        return False

def mywalk(top, skipdirs=['.snapshot',]):
    """ returns subset of os.walk  """
    for root, dirs, files in walk(top,topdown=True,onerror=walkerr): 
        for skipdir in skipdirs:
            if skipdir in dirs:
                dirs.remove(skipdir)  # don't visit this directory 
        yield root, dirs, files 

def walkerr(oserr):    
    sys.stderr.write(str(oserr))
    sys.stderr.write('\n')
    return 0


def send_mail(to, subject, text, attachments=[], cc=[], bcc=[], smtphost="", fromaddr=""):

    if sys.version_info[0] == 2:
        from email.MIMEMultipart import MIMEMultipart
        from email.MIMEBase import MIMEBase
        from email.MIMEText import MIMEText
        from email.Utils import COMMASPACE, formatdate
        from email import Encoders
    else:
        from email.mime.multipart import MIMEMultipart
        from email.mime.base import MIMEBase
        from email.mime.text import MIMEText
        from email.utils import COMMASPACE, formatdate
        from email import encoders as Encoders
    from string import Template
    import socket
    import smtplib

    if not isinstance(to,list):
        print("the 'to' parameter needs to be a list")
        return False    
    if len(to)==0:
        print("no 'to' email addresses")
        return False
    
    myhost=socket.getfqdn()

    if smtphost == '':
        smtphost = get_mx_from_email_or_fqdn(myhost)
    if not smtphost:
        sys.stderr.write('could not determine smtp mail host !\n')
        
    if fromaddr == '':
        fromaddr = os.path.basename(__file__) + '-no-reply@' + \
           '.'.join(myhost.split(".")[-2:]) #extract domain from host
    tc=0
    for t in to:
        if '@' not in t:
            # if no email domain given use domain from local host
            to[tc]=t + '@' + '.'.join(myhost.split(".")[-2:])
        tc+=1

    message = MIMEMultipart()
    message['From'] = fromaddr
    message['To'] = COMMASPACE.join(to)
    message['Date'] = formatdate(localtime=True)
    message['Subject'] = subject
    message['Cc'] = COMMASPACE.join(cc)
    message['Bcc'] = COMMASPACE.join(bcc)

    body = Template('This is a notification message from $application, running on \n' + \
            'host $host. Please review the following message:\n\n' + \
            '$notify_text\n\nIf output is being captured, you may find additional\n' + \
            'information in your logs.\n'
            )
    host_name = socket.gethostname()
    full_body = body.substitute(host=host_name.upper(), notify_text=text, application=os.path.basename(__file__))

    message.attach(MIMEText(full_body))

    for f in attachments:
        part = MIMEBase('application', 'octet-stream')
        part.set_payload(open(f, 'rb').read())
        Encoders.encode_base64(part)
        part.add_header('Content-Disposition', 'attachment; filename="%s"' % os.path.basename(f))
        message.attach(part)

    addresses = []
    for x in to:
        addresses.append(x)
    for x in cc:
        addresses.append(x)
    for x in bcc:
        addresses.append(x)

    smtp = smtplib.SMTP(smtphost)
    smtp.sendmail(fromaddr, addresses, message.as_string())
    smtp.close()

    return True

def get_mx_from_email_or_fqdn(addr):
    """retrieve the first mail exchanger dns name from an email address."""
    # Match the mail exchanger line in nslookup output.
    MX = re.compile(r'^.*\s+mail exchanger = (?P<priority>\d+) (?P<host>\S+)\s*$')
    # Find mail exchanger of this email address or the current host
    if '@' in addr:
        domain = addr.rsplit('@', 2)[1]
    else:
        domain = '.'.join(addr.rsplit('.')[-2:])
    p = os.popen('/usr/bin/nslookup -q=mx %s' % domain, 'r')
    mxes = list()
    for line in p:
        m = MX.match(line)
        if m is not None:
            mxes.append(m.group('host')[:-1])  #[:-1] just strips the ending dot
    if len(mxes) == 0:
        return ''
    else:
        return mxes[0]

def parse_arguments():
    """
    Gather command-line arguments.
    """
    parser = argparse.ArgumentParser(prog='swsymlinks',
        description='save or restore all symlinks in a directory structure' +\
               'to a textfile .symbolic-links.tree.txt')
    parser.add_argument( '--save', '-s', dest='save', action='store_true',
        help='save all symlinks to a text file',
        default=False )
    parser.add_argument( '--single', '-n', dest='single', action='store_true',
        help='save all symlinks into a single file .symbolic-links.tree.txt at the root',
        default=False )         
    parser.add_argument( '--clean', '-c', dest='clean', action='store_true',
        help='delete all .symbolic-links.tree.txt files recursively',
        default=False ) 
    parser.add_argument( '--restore', '-r', dest='restore', action='store_true',
        help='restore all symlinks from a text file',
        default=False )
    parser.add_argument( '--force', '-o', dest='force', action='store_true',
        help='force restore, even if a symlink already exists.',
        default=False )    
    parser.add_argument( '--debug', '-g', dest='debug', action='store_true',
        help='print the symlink targets to STDERR',
        default=False )        
    parser.add_argument( '--email-notify', '-e', dest='email',
        action='store',
        help='notify this email address of any error ',
        default='' )
    parser.add_argument( '--linkdbfolder', '-l', dest='linkdbfolder',
        action='store', 
        help='set folder location for .symbolic-links.tree.txt file')    
    parser.add_argument( '--folder', '-f', dest='folder',
        action='store', 
        help='search this folder and below for files to remove')
    args = parser.parse_args()
    if not args.folder:
        parser.error('required option --folder not given !')
    if args.debug:
        print('DEBUG: Arguments/Options: %s' % args)    
    return args

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3

# get multisegment swift files in parallel

import sys,os,getopt,json
import time

import multiprocessing

import swiftclient

def create_sparse_file(filename,length):
   #print("creating sparse file",filename)
   with open(filename, "wb") as f:
      f.truncate(length)

swift_auth_token=os.environ.get("OS_AUTH_TOKEN")
storage_url=os.environ.get("OS_STORAGE_URL")

def create_sw_conn():
   global swift_auth_token,storage_url

   if swift_auth_token and storage_url:
      return swiftclient.Connection(preauthtoken=swift_auth_token,
         preauthurl=storage_url)

   swift_auth=os.environ.get("ST_AUTH")
   swift_user=os.environ.get("ST_USER")
   swift_key=os.environ.get("ST_KEY")

   if swift_auth and swift_user and swift_key:
      return swiftclient.Connection(authurl=swift_auth,user=swift_user,
         key=swift_key)

   print("Error: Swift environment not configured!")
   sys.exit()

def parseSwiftUrl(path):
    path = path.lstrip('/')
    components = path.split('/');
    container = components[0];
    obj = '/'.join(components[1:])
    return container, obj

# container, object, offset, dest
def assemble_ms_object(x):
   #print("assembling",x) 
   conn=create_sw_conn()

   headers,body=conn.get_object(x[0],x[1])
   #print("body len=",len(body))
   with open(x[3],"r+b") as f_out:
      if x[2]>0:
         f_out.seek(x[2])
      f_out.write(bytes(body))

   conn.close()

def get_ms_object(sc,container,object,pool_size):
   #print("multisegment object",object)
   segments=[]
   segment_total=0

   # build segment map for parallel download
   headers,body=sc.get_object(container,object,
      query_string='multipart-manifest=get')
   manifest=json.loads(body.decode())
   dest=os.path.basename(object)
   for segment in manifest:
      segment_container,segment_obj=parseSwiftUrl(segment['name'])
      # store segment container, object and offset
      segments.append([segment_container,segment_obj,segment_total,dest])
      segment_total=segment_total+segment['bytes']

   # create sparse file
   create_sparse_file(dest,segment_total)

   # sequential assembly
   #for seg in segments:
   #   assemble_ms_object(seg)

   # parallel assembly
   #print("parallel assembly with",pool_size,"workers")
   p=multiprocessing.Pool(pool_size)
   p.map(assemble_ms_object,segments)

def get_object(conn,container,object):
   headers,body=conn.get_object(container,object)
   with open(os.path.basename(object),"w+b") as f_out:
      f_out.write(bytes(body))

def set_time(headers,name):
   if headers:
      if 'x-object-meta-mtime' in headers:
         mmt=int(float(headers['x-object-meta-mtime']))
      else:
         mkt=time.mktime(time.strptime(headers['last-modified'],
            "%a, %d %b %Y %X %Z"))
         mmt=int(time.mktime(time.localtime(mkt)))

      os.utime(os.path.basename(name),(mmt,mmt))

def get_objects(sc,container,object_list,pool_size):
   found=0
   #print("getting",object_list,"from container",container)

   try:
      headers,objs=sc.get_container(container,full_listing=True)
      for obj in objs:
         #print("found",obj['name'])
         if obj['name'] in object_list:
            #print("matched",obj['name'])
            found=found+1
            try:
               headers=sc.head_object(container, obj['name'])
            except:
               headers=[]
           
            if 'x-static-large-object' in headers:
               get_ms_object(sc,container,obj['name'],pool_size)
            else:
               get_object(sc,container,obj['name'])

            set_time(headers,obj['name'])

      if not found:
         print("No matching files found")

   except swiftclient.ClientException:
      print("Error: cannot access Swift container '%s'!" % container)

   sc.close()

def validate_dir(path,param):
   if not os.path.isdir(path):
      print("Error: %s '%s' is not accessible!" % (param,path))
      sys.exit()

   if path[-1]=='/':
      path=path[:-1] 

   return(path)

def usage():
   print("swpget [parameters]")
   print("Parameters:")
   print("\t-l local_directory (default .)")
   print("\t-c container (required)")
   print("\t-p pool_size (default 5)")
   print("\t-a auth_token (default OS_AUTH_TOKEN)")
   print("\t-s storage_url (default OS_STORAGE_URL)")

def main(argv=None):
   global swift_auth_token
   global storage_url

   argv = argv or sys.argv[1:]

   container=""
   pool_size=5

   try:
      opts,args=getopt.getopt(argv,"l:c:p:a:s:h")
   except getopt.GetoptError:
      usage()
      sys.exit()

   for opt,arg in opts:
      if opt in ("-h"):
         container=""
         break
      elif opt in ("-l"): # override default local directory
         local_dir=validate_dir(arg,"local")
         os.chdir(local_dir)
      elif opt in ("-c"): # set container
         container=arg
      elif opt in ("-p"): # parallel workers
         pool_size=int(arg)
      elif opt in ("-a"): # override swift_auth_token
         swift_auth_token=arg
      elif opt in ("-s"): # override storage URL
         storage_url=arg

   if not container or not args:
      usage()
   else:
      sc=create_sw_conn()
      if sc:
         get_objects(sc,container,args,pool_size)

if __name__ == '__main__':
   main()



#! /usr/bin/env python3

# Script to mass delete swift objects in a pseudo folder
#
# swrm.py dirkpetersen / Jul 2015 
#

import swiftclient, sys, os, math, argparse, json

class KeyboardInterruptError(Exception): pass

def main():
    global args

    # Parse command-line arguments
    args = parse_arguments()
    
    if args.container:

        c=create_sw_conn()

        storageurl=os.environ.get("OS_STORAGE_URL")
        if not storageurl:
            storageurl, authtoken = c.get_auth()
        
        swaccount=storageurl.split("/")[-1]

        prefix=args.prefix
        if prefix:
            if prefix.endswith('*'):
                prefix=args.prefix[:-1]
            elif not prefix.endswith('/'):
                # may be single object, try deleting
                obj = {}
                obj['name']=args.prefix
                ret=delobj(obj)
                if ret==200:
                    return True
                elif ret==404:
                    prefix=args.prefix+'/'
                else:
                    print("Error %s deleting object %s" % (ret,args.prefix))
        else:
            print('Warning: no prefix / pseudo folder entered - will delete container')
        
        print ("    checking swift folder /%s/%s ..." % (args.container,prefix))
        try:
            headers, objects = c.get_container(args.container,prefix=prefix,full_listing=True)
            c.close()
        except swiftclient.client.ClientException as ex:
            httperr=getattr(ex, 'http_status', None)
            if httperr == 404:
                print("    no objects found - error 404")
            else:
                print("    HTTP Error %s" % httperr)
            return False

        sbytes=0
        print("    checking size of /%s/%s ... " % (args.container,prefix))
        for obj in objects:
            sbytes+=obj['bytes']
            #print(obj['name'],obj['bytes'])
        if sbytes > 0:
            print ("        found %s files with %s bytes (%s) in /%s/%s" % (len(objects),intwithcommas(sbytes),
                  convertByteSize(sbytes),args.container,prefix))
            if not args.force:
                if yn_choice("    Do you want to delete this data in account %s ?" % swaccount):
                    easy_par(delobj,objects)
            else:
                easy_par(delobj,objects)
        else:
            if prefix:
                print ("    Error: it seems swift objects at /%s/%s do not exist" % (args.container,args.prefix))
            else:
                c=create_sw_conn()
                c.delete_container(args.container)
                c.close()
    else:
        print('no container entered - aborting')
        return False


def delobj(obj):
    print("    deleting /%s/%s ... " % (args.container,obj['name']))
    retcode=200
    c=create_sw_conn()
    try:
        headers = c.head_object(args.container, obj['name'])
    except:
        headers = []
    if 'x-static-large-object' in headers:
        #print(headers['x-static-large-object'])
        headers, body = c.get_object(args.container, obj['name'], query_string='multipart-manifest=get')
        manifest = json.loads(body.decode())
        for segment in manifest:
            print ("        deleting segment %s" % segment['name'])
            segment_container, segment_object = parseSwiftUrl(segment['name'])
            try:
            	c.delete_object(segment_container, segment_object)
            except Exception as e:
                print('Error %s deleting object segment %s: %r' % (e.http_status,obj['name'],e))
    try:
        c.delete_object(args.container,obj['name'])
    except Exception as e:
        retcode=e.http_status
        print('Error %s deleting object %s: %r' % (e.http_status,obj['name'],e))
    c.close()
    return retcode

def parseSwiftUrl(path):
    path = path.lstrip('/')
    components = path.split('/');
    container = components[0];
    obj = '/'.join(components[1:])
    return container, obj
    
def yn_choice(message, default='n'):
    choices = 'Y/n' if default.lower() in ('y', 'yes') else 'y/N'
    choice = input("%s (%s) " % (message, choices))
    values = ('y', 'yes', '') if default == 'y' else ('y', 'yes')
    return choice.strip().lower() in values

def easy_par(f, sequence):    
    from multiprocessing import Pool
    pool = Pool(processes=args.maxproc)
    try:
        # f is given sequence. guaranteed to be in order
        cleaned=False
        result = pool.map(f, sequence)
        cleaned = [x for x in result if not x is None]
        #cleaned = asarray(cleaned)
        # not optimal but safe
    except KeyboardInterrupt:
        pool.terminate()
    except Exception as e:
        print('got exception: %r' % (e,))
        if not args.force:
            print("Terminating the pool")
            pool.terminate()
    finally:
        pool.close()
        pool.join()
        return cleaned

def create_sw_conn():
    if args.authtoken and args.storageurl:
        return swiftclient.Connection(preauthtoken=args.authtoken, preauthurl=args.storageurl)
    else:
        authtoken=os.environ.get("OS_AUTH_TOKEN")
        storageurl=os.environ.get("OS_STORAGE_URL")
        if authtoken and storageurl:
            return swiftclient.Connection(preauthtoken=authtoken, preauthurl=storageurl)
        else:
            swift_auth=os.environ.get("ST_AUTH")
            swift_user=os.environ.get("ST_USER")
            swift_key=os.environ.get("ST_KEY")
            if swift_auth and swift_user and swift_key:
                return swiftclient.Connection(authurl=swift_auth,user=swift_user,key=swift_key)

def convertByteSize(size):
   if size == 0:
        return '0 B'
   size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
   i = int(math.floor(math.log(size,1024)))
   p = math.pow(1024,i)
   s = size/p
   if (s > 0):
       return '%0.3f %s' % (s,size_name[i])
   else:
       return '0 B'

def intwithcommas(x):
    result=''
    while x >= 1000:
        x,r = divmod(x, 1000)
        result = ",%03d%s" % (r, result)
    return "%d%s" % (x, result)


def parse_arguments():
    """
    Gather command-line arguments.
    """
    parser = argparse.ArgumentParser(
        description='delete a pseudo folder in swift with ' + \
        'potentially many objects ' + \
        '()')
    parser.add_argument( '--container', '-c', dest='container',
        action='store',
        help='a container in the swift object store',
        default='' )
    parser.add_argument( '--prefix', '-p', dest='prefix',
        action='store',
        help='a swift object prefix',
        default=None)
    parser.add_argument( '--force', '-f', dest='force',
        action='store_true',
        help='force delete without prompt',
        default=False)
    parser.add_argument( '--maxproc', '-m', dest='maxproc',
        action='store',
        type=int,
        help='maximum number of processes to run',
        default=32 )
    parser.add_argument( '--authtoken', '-a', dest='authtoken',
        action='store',
        help='a swift authentication token (required when storage-url is used)',
        default=None)    
    parser.add_argument( '--storage-url', '-s', dest='storageurl',
        action='store',
        help='a swift storage url (required when authtoken is used)',
        default=None)
    
    args = parser.parse_args()
    return args

if __name__ == '__main__':
    sys.exit(main())




#! /usr/bin/env python3

# Script for comparing the size of a posix folder with the size of a swift pseudo folder
#
# swfoldersize.py dirkpetersen / Jan 2015 
#

import swiftclient, sys, os, argparse, math, functools

class KeyboardInterruptError(Exception): pass

def main():
    global args
    # Parse command-line arguments
    args = parse_arguments()

    if args.container:
        c=create_sw_conn()
        sbytes=0
        print ("    checking swift folder %s/%s ..." % (args.container,args.prefix))
        try:
            headers, objects = c.get_container(args.container,prefix=args.prefix,full_listing=True)            
            for obj in objects:
                sbytes+=obj['bytes']
                #print(obj['name'],obj['bytes'])
            if sbytes > 0:
                print ("    %s bytes (%s) in %s/%s (swift)" % (intwithcommas(sbytes),convertByteSize(sbytes),args.container,args.prefix))
            else:
                print ("    ...Error: it seems swift folder %s/%s does not contain any data." % (args.container,args.prefix))
        except:
            print ("    ...Error: it seems swift folder %s/%s does not exist" % (args.container,args.prefix))

    if args.posixfolder:
        print ("    checking posix folder %s ..." % (args.posixfolder))
        pbytes = getFolderSize(os.path.expanduser(args.posixfolder))
        print ("    %s bytes (%s) in %s" % (intwithcommas(pbytes),convertByteSize(pbytes),args.posixfolder))
        
    if args.posixfolder2:
        print ("    checking 2nd posix folder %s ..." % (args.posixfolder2))
        p2bytes = getFolderSize(os.path.expanduser(args.posixfolder2))
        print ("    %s bytes (%s) in %s" % (intwithcommas(p2bytes),convertByteSize(p2bytes),args.posixfolder2))

    if args.posixfolder and args.container:
        if sbytes == pbytes:
            print("OK! The size of %s and %s/%s is identical!" % \
                    (args.posixfolder,args.container,args.prefix))
        else:
            print("********** WARNING !! ********** The size of  %s and %s/%s is NOT identical!" % \
                    (args.posixfolder,args.container,args.prefix))
                    
    if args.posixfolder and args.posixfolder2:
        if p2bytes == pbytes:
            print("OK! The size of %s and %s is identical!" % \
                    (args.posixfolder,args.posixfolder2))
        else:
            print("********** WARNING !! ********** The size of  %s and %s is NOT identical!" % \
                    (args.posixfolder,args.posixfolder2))                    

def getFolderSize(p):
    if "/.snapshot/" in p:
        return 0
    if os.path.islink(p):
        return 0
    prepend = functools.partial(os.path.join, p)
    try:
        return sum([(os.path.getsize(f) if not os.path.islink(f) and os.path.isfile(f) else getFolderSize(f)) for f in map(prepend, os.listdir(p))])
    except:
        print("    ...Error getting size of folder %s" % p)
        return 0

def create_sw_conn():
    if args.authtoken and args.storageurl:
        return swiftclient.Connection(preauthtoken=args.authtoken, preauthurl=args.storageurl)
    else:
        authtoken=os.environ.get("OS_AUTH_TOKEN")
        storageurl=os.environ.get("OS_STORAGE_URL")
        if authtoken and storageurl:
            return swiftclient.Connection(preauthtoken=authtoken, preauthurl=storageurl)
        else:
            swift_auth=os.environ.get("ST_AUTH")
            swift_user=os.environ.get("ST_USER")
            swift_key=os.environ.get("ST_KEY")
            if swift_auth and swift_user and swift_key:
                return swiftclient.Connection(authurl=swift_auth,user=swift_user,key=swift_key)

def convertByteSize(size):
   if size == 0:
   	return '0 B'
   size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
   i = int(math.floor(math.log(size,1024)))
   p = math.pow(1024,i)
   s = size/p
   if (s > 0):
       return '%0.3f %s' % (s,size_name[i])
   else:
       return '0 B'

def intwithcommas(x):
    result=''
    while x >= 1000:
        x,r = divmod(x, 1000)
        result = ",%03d%s" % (r, result)
    return "%d%s" % (x, result)

def parse_arguments():
    """
    Gather command-line arguments.
    """
    parser = argparse.ArgumentParser(prog='swfoldersize.py',
        description='compare the size of a posix folder with the size ' + \
        'of a swift (pseudo) folder after a data migration ' + \
        '()')
    parser.add_argument( '--posixfolder', '-p', dest='posixfolder',
        action='store',
        help='a folder on a posix file system ',
        default='' )        
    parser.add_argument( '--posixfolder2', '-2', dest='posixfolder2',
        action='store',
        help='a 2nd folder on a posix file system ',
        default='' )        
    parser.add_argument( '--container', '-c', dest='container',
        action='store',
        help='a container in the swift object store',
        default='' )
    parser.add_argument( '--prefix', '-x', dest='prefix',
        action='store',
        help='a swift object prefix',
        default=None)
    parser.add_argument( '--proc', '-m', dest='maxproc',
        action='store',
        type=int,
        help='maximum number of processes to run (not yet implemented)',
        default=0 )
    parser.add_argument( '--authtoken', '-a', dest='authtoken',
        action='store',
        help='a swift authentication token (required when storage-url is used)',
        default=None)
    parser.add_argument( '--storage-url', '-s', dest='storageurl',
        action='store',
        help='a swift storage url (required when authtoken is used)',
        default=None)

    args = parser.parse_args()
    return args

if __name__ == '__main__':
    sys.exit(main())




#!/usr/bin/env python3

import os,sys,getopt,tarfile
import getpass

from distutils.spawn import find_executable

import time
import socket
import optparse
import subprocess
import multiprocessing

from swiftclient import Connection

from swiftclient import shell
from swiftclient import RequestException
from swiftclient.exceptions import ClientException
from swiftclient.multithreading import OutputManager

try:
    from scandir import walk
except:
    from os import walk

swift_auth=os.environ.get("ST_AUTH")
swift_auth_token=os.environ.get("OS_AUTH_TOKEN")
storage_url=os.environ.get("OS_STORAGE_URL")
haz_pigz=False

# define minimum parser object to allow swiftstack shell to run 
def shell_minimal_options():
   global swift_auth,swift_auth_token,storage_url

   parser = optparse.OptionParser()

   parser.add_option('-A', '--auth', dest='auth',
      default=swift_auth)
   parser.add_option('-V', '--auth-version',
      default=os.environ.get('ST_AUTH_VERSION',
         (os.environ.get('OS_AUTH_VERSION','1.0'))))
   parser.add_option('-U', '--user', dest='user',
      default=os.environ.get('ST_USER'))
   parser.add_option('-K', '--key', dest='key',
      default=os.environ.get('ST_KEY'))

   parser.add_option('--os_auth_token',default=swift_auth_token)
   parser.add_option('--os_storage_url',default=storage_url)

   parser.add_option('--os_username')
   parser.add_option('--os_password')
   parser.add_option('--os_auth_url')

   parser.add_option('--os_user_id')
   parser.add_option('--os_user_domain_id')
   parser.add_option('--os_user_domain_name')
   parser.add_option('--os_tenant_id')
   parser.add_option('--os_tenant_name')
   parser.add_option('--os_project_id')
   parser.add_option('--os_project_domain_id')
   parser.add_option('--os_project_name')
   parser.add_option('--os_project_domain_name')
   parser.add_option('--os_service_type')
   parser.add_option('--os_endpoint_type')
   parser.add_option('--os_region_name')

   # new mandatory bogosity required for swiftclient >= 3.0.0
   parser.add_option('--debug')
   parser.add_option('--info')
   
   parser.add_option('-v', '--verbose', action='count', dest='verbose',
       default=1, help='Print more info.')

   return parser

# wrapper function for swiftstack shell functions
def sw_shell(sw_fun,*args):
   global swift_auth_token,storage_url

   if swift_auth_token and storage_url:
      args=args+("--os_auth_token",swift_auth_token,
         "--os_storage_url",storage_url)

   args = ('',) + args
   with OutputManager() as output:
      parser = shell_minimal_options()
      try:
         sw_fun(parser, list(args), output)
      except (ClientException, RequestException, socket.error) as err:
         output.error(str(err))
 
def sw_download(*args):
   sw_shell(shell.st_download,*args)
 
def sw_upload(*args):
   sw_shell(shell.st_upload,*args)
 
def sw_post(*args):
   sw_shell(shell.st_post,*args)
 
# suffix of archive files
tar_suffix=".tar.gz"
root_id=".root"

# True if 1st char of path member is '.' else False
def is_hidden_dir(dir_name):
   for item in dir_name.split('/'):
      if item[0]=='.':
         return True

   return False

def print_flush(str):
   sys.stdout.write(str+'\n')
   sys.stdout.flush()

# apparently getpid is ok because it's different between mp tasks
def unique_id():
   return str(os.getpid())

def create_tar_file(filename,src_path,file_list):
   global haz_pigz

   # only archive src_path directory
   tar_params=["tar","cvf",filename,"--directory="+src_path,"--no-recursion"]
   if haz_pigz:
      tar_params=tar_params+["--use-compress-program=pigz"]

   # include directory itself in archive for ownership & permissions
   tar_params=tar_params+['.']

   # generate external file list only if files to be archived
   if file_list:
      tmp_file="/tmp/.tar."+unique_id()
      with open(tmp_file,"w") as f:
         for file in file_list:
            f.write("-- \""+file+"\"\n")
      tar_params=tar_params+["-T",tmp_file]
  
   ret=subprocess.call(tar_params)
   if ret>0:
      sys.stderr.write('***** TAR ERROR %s, command: %s *****\n' % 
         (ret,tar_params))   

   if file_list:
      os.unlink(tmp_file)

def upload_file_to_swift(filename,swiftname,container,meta):
   final=[container,filename]
   if meta:
      final=meta+final

   sw_upload("--object-name="+swiftname,
      "--segment-size=2147483648",
      "--use-slo",
      "--segment-container=.segments_"+container,
      "--header=X-Object-Meta-Uploaded-by:"+getpass.getuser(),*final)

def archive_tar_file(src_path,file_list,container,tmp_dir,pre_path,meta):
   global tar_suffix

   # archive_name is name for archived object
   archive_name=pre_path+tar_suffix
   # temp_archive_name is name of local tar file
   temp_archive_name=unique_id()+os.path.basename(archive_name)
   if tmp_dir:
      temp_archive_name=os.path.join(tmp_dir,temp_archive_name)
  
   # Create local tar file 
   create_tar_file(temp_archive_name,src_path,file_list)

   # Upload tar file to container as 'archive_name' 
   upload_file_to_swift(temp_archive_name,archive_name,container,meta)

   # Delete local tar file
   os.unlink(temp_archive_name)

def is_child_or_sib(dir_name,last_dir):
   dname=os.path.dirname(dir_name) 
   return (dname==last_dir or dname==os.path.dirname(last_dir))

# param order: [src_path,file_list,container,tmp_dir,pre_path,meta]
def archive_worker(item):
   archive_tar_file(*item)

def archive_to_swift(local_dir,container,no_hidden,tmp_dir,prefix,par,subtree,
   meta):
   last_dir=""

   # Now updating object not container metadata
   #sw_post(container,*meta)
   sw_post(container)

   archive_pool=multiprocessing.Pool(par)

   for dir_name, subdir_list, file_list in mywalk(local_dir):
      rel_path=os.path.relpath(dir_name,local_dir)
      if (not (no_hidden and is_hidden_dir(rel_path))):
         # if files in root directory use basename of root
         if rel_path==".":
            rel_path=os.path.basename(dir_name)+root_id

         if (not subtree) or (is_subtree(subtree,dir_name)):
            archive_pool.apply_async(archive_worker,
               [[dir_name,file_list,container,tmp_dir,
                  os.path.join(prefix,rel_path),meta]])

         last_dir=dir_name

   archive_pool.close()
   archive_pool.join()

# parse name into directory tree
def create_local_path(local_dir,archive_name):
   global tar_suffix

   path=os.path.join(local_dir,archive_name)
   if path.endswith(tar_suffix):
      path=path[:-len(tar_suffix)]

   if not os.path.exists(path):
      os.makedirs(path)
   
   return path

def create_sw_conn():
   global swift_auth,swift_auth_token,storage_url

   if swift_auth_token and storage_url:
      return Connection(preauthtoken=swift_auth_token,preauthurl=storage_url)

   if swift_auth:
      swift_user=os.environ.get("ST_USER")
      swift_key=os.environ.get("ST_KEY")

      if swift_user and swift_key:
         return Connection(authurl=swift_auth,user=swift_user,key=swift_key)

   print("Error: Swift environment not configured!")
   sys.exit()

def extract_tar_file(tarfile,termpath):
   global haz_pigz

   tar_params=["tar","xvf",tarfile,"--directory="+termpath]
   if haz_pigz:
      tar_params=tar_params+["--use-compress-program=pigz"]

   ret=subprocess.call(tar_params)
   if ret > 0:
      sys.stderr.write('***** TAR ERROR %s, command: %s *****\n' % 
         (ret,tar_params))
   
def retrieve_tar_file(tmp_dir,container,obj_name,local_dir,prefix):
   global tar_suffix
   global root_id

   # download tar file and extract into terminal directory
   temp_file=unique_id()+tar_suffix
   if tmp_dir:
      temp_file=os.path.join(tmp_dir,temp_file)

   sw_download("--output="+temp_file,container,obj_name)

   # strip prefix and if next char is /, strip it too
   if prefix and obj_name.startswith(prefix):
      obj_name=obj_name[len(prefix):] 
      if obj_name[0]=='/':
         obj_name=obj_name[1:]
   
   # if bundle, extract using tar embedded paths
   if obj_name.endswith(root_id+tar_suffix):
      term_path=local_dir
   else:
      term_path=create_local_path(local_dir,obj_name)

   extract_tar_file(temp_file,term_path)

   os.unlink(temp_file)

def extract_worker(item):
   retrieve_tar_file(*item)

def extract_to_local(local_dir,container,no_hidden,tmp_dir,prefix,par):
   global tar_suffix
   global root_id

   swift_conn=create_sw_conn()
   if swift_conn:
      try: 
         headers,objs=swift_conn.get_container(container,prefix=prefix,
            full_listing=True)

         extract_pool=multiprocessing.Pool(par)

         for obj in objs:
            if obj['name'].endswith(tar_suffix):
               if no_hidden and is_hidden_dir(obj['name']):
                  continue

               # param order: [tmp_dir,container,obj_name,local_dir,prefix]
               extract_pool.apply_async(extract_worker,
                  [[tmp_dir,container,obj['name'],local_dir,prefix]])

         extract_pool.close()
         extract_pool.join()
      except ClientException:
         print("Error: cannot access Swift container '%s'!" % container)

      swift_conn.close()

def usage():
   print("archive [parameters]")
   print("Parameters:")
   print("\t-l local_directory (default .)")
   print("\t-c container (required)")
   print("\t-x (extract from container to local directory)")
   print("\t-n (no hidden directories)")
   print("\t-t temp_dir (directory for temp files)")
   print("\t-a auth_token (default OS_AUTH_TOKEN)")
   print("\t-s storage_url (default OS_STORAGE_URL)")
   print("\t-p prefix")
   print("\t-P parallel_instances (default 3)")
   print("\t-m name:value (set object metadata)")

# is path a child of tree?
def is_subtree(tree,path):
   tree_sp=tree.split('/')
   path_sp=path.split('/')

   if len(path_sp)<len(tree_sp):
      return False

   for t,p in zip(tree_sp,path_sp):
      if t!=p:
         return False

   return True

def validate_dir(path,param):
   if not os.path.isdir(path):
      print("Error: %s '%s' is not accessible!" % (param,path))
      sys.exit()

   if path[-1]=='/':
      path=path[:-1] 

   return(path)

def mywalk(top, skipdirs=['.snapshot',]):
    """ returns subset of os.walk  """
    for root, dirs, files in walk(top,topdown=True,onerror=walkerr):
        for skipdir in skipdirs:
            if skipdir in dirs:
                dirs.remove(skipdir)  # don't visit this directory 
        yield root, dirs, files

def walkerr(oserr):
    sys.stderr.write(str(oserr))
    sys.stderr.write('\n')
    return 0

def main(argv=None):
   global swift_auth_token
   global storage_url
   global haz_pigz
   argv = argv or sys.argv[1:]

   meta=[]
   sub_tree=""
   local_dir="."
   container=""
   tmp_dir=""
   extract=False
   no_hidden=False
   prefix=""
   par=3

   try:
      opts,args=getopt.getopt(argv,"l:c:t:a:s:p:P:S:m:xnh")
   except getopt.GetoptError:
      usage()
      sys.exit()

   for opt,arg in opts:
      if opt in ("-h"):
         usage()
         sys.exit()
      elif opt in ("-l"): # override default local directory
         local_dir=validate_dir(arg,"local")
      elif opt in ("-c"): # set container
         container=arg
      elif opt in ("-t"): # temp file directory
         tmp_dir=validate_dir(arg,"tmp_dir")
      elif opt in ("-a"): # override auth_token
         swift_auth_token=arg
      elif opt in ("-s"): # override storage URL
         storage_url=arg
      elif opt in ("-p"): # set prefix
         prefix=arg
      elif opt in ("-P"): # set parallel threads
         par=int(arg)
      elif opt in ("-x"): # extract mode
         extract=True
      elif opt in ("-n"): # set no-hidden flag to skip .*
         no_hidden=True
      elif opt in ("-S"): # specify optional sub-tree
         sub_tree=validate_dir(os.path.join(local_dir,arg),"subtree")
      elif opt in ("-m"): # specify object metadata
         if arg.count(':')==1:
            meta.append("-HX-Object-Meta-"+arg)
         else:
            print("Error: metadata not in format key:value!")
            sys.exit()

   if not container:
      usage()
   else:
      if find_executable("pigz"):
         haz_pigz=True

      if extract:
         extract_to_local(local_dir,container,no_hidden,tmp_dir,prefix,par)
      else:
         archive_to_swift(local_dir,container,no_hidden,tmp_dir,prefix,par,
            sub_tree,meta)

if __name__=="__main__":
   main()



from setuptools import setup

__version__ = "1.4.2"

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

CLASSIFIERS = [
    "Development Status :: 5 - Production/Stable",
    "Environment :: Console",
    "Environment :: OpenStack",
    "Intended Audience :: Customer Service",
    "Intended Audience :: Developers",
    "Intended Audience :: Education",
    "Intended Audience :: End Users/Desktop",
    "Intended Audience :: Healthcare Industry",
    "Intended Audience :: Information Technology",
    "Intended Audience :: Science/Research",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: Apache Software License",
    "Natural Language :: English",
    "Operating System :: MacOS :: MacOS X",
    "Operating System :: POSIX",
    "Operating System :: POSIX :: Linux",
    "Operating System :: POSIX :: Other",
    "Operating System :: Unix",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Programming Language :: Unix Shell",
    "Topic :: Desktop Environment :: File Managers",
    "Topic :: Internet",
    "Topic :: Scientific/Engineering :: Bio-Informatics",
    "Topic :: System :: Archiving",
    "Topic :: System :: Archiving :: Backup",
    "Topic :: System :: Filesystems",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: System :: Systems Administration",
    "Topic :: Utilities"
]

setup(
    name='swift-commander',
    version=__version__,
    description='''\
swift commander (swc) is a wrapper to various command line
client tools for openstack swift cloud storage systems.''',
    long_description=read_md('README.md'),
    packages=['swift_commander'],
    scripts=['swift_commander/swc'],
    author = 'Dirk Petersen, Jeff Katcher',
    author_email = 'dp@nowhere.com',
    url = 'https://github.com/FredHutch/swift-commander', 
    download_url = 'https://github.com/FredHutch/swift-commander/tarball/%s' % __version__,
    keywords = ['openstack', 'swift', 'cloud storage'], # arbitrary keywords
    classifiers = CLASSIFIERS,
    # 'python-swiftclient>=2.5,<3','python-keystoneclient>=1.5,<2'
    install_requires=[
        'psutil>=4',
        'python-swiftclient>=3,<4',
        'python-keystoneclient>=2,<3'
        ],
    entry_points={
        # we use console_scripts here to allow virtualenv to rewrite shebangs
        # to point to appropriate python and allow experimental python 2.X
        # support.
        'console_scripts': [
            'swbundler.py=swift_commander.swbundler:main',
            'swfoldersize.py=swift_commander.swfoldersize:main',
            'swhashcomp.py=swift_commander.swhashcomp:main',
            'swpget.py=swift_commander.swpget:main',
            'swrm.py=swift_commander.swrm:main',
            'swsearch.py=swift_commander.swsearch:main',
            'swsymlinks.py=swift_commander.swsymlinks:main',
        ]
    }
)



#!/usr/bin/env python3

# get multisegment swift files in parallel

import re
import argparse
import fnmatch
import psutil

import sys,os,getopt,json
import time

import multiprocessing

import swiftclient

def create_sw_conn(swift_auth_token,storage_url):
    if swift_auth_token and storage_url:
        return swiftclient.Connection(preauthtoken=swift_auth_token,
            preauthurl=storage_url)

    swift_auth=os.environ.get("ST_AUTH")
    swift_user=os.environ.get("ST_USER")
    swift_key=os.environ.get("ST_KEY")

    if swift_auth and swift_user and swift_key:
        return swiftclient.Connection(authurl=swift_auth,user=swift_user,
            key=swift_key)

    print("Error: Swift environment not configured!")
    sys.exit()

textchars=bytearray({7,8,9,10,12,13,27}|set(range(0x20, 0x100))-{0x7f})
is_binary_string=lambda bytes:bool(bytes.translate(None,textchars))

def print_match(object,body,pattern,range=25):
    offset=body.find(pattern)
    if offset!=-1:
        print("%s: matched at offset %d" % (object,offset),flush=True)
        excerpt=body[max(0,offset-range):offset+len(pattern)+range]
        print('\t'+repr(excerpt.decode(encoding="ISO-8859-1")))

def search_object(parse_arg,object):
    sc=create_sw_conn(parse_arg.authtoken,parse_arg.storage_url)

    match=object.find(parse_arg.pattern)
    if match!=-1:
       print("%s: matched object name" % object,flush=True)

    headers,body=sc.get_object(parse_arg.container,object)
    if not parse_arg.binary or not is_binary_string(body):
        #print("scanning object",object,flush=True)
        if not parse_arg.insensitive:
            print_match(object,body,bytes(parse_arg.pattern,"ISO-8859-1"))
        else:
            m_o=re.search(bytes(parse_arg.pattern,"utf-8"),body,re.IGNORECASE)
            if m_o:
                print_match(object,body,m_o.group(0))

    sc.close()

# order is type,sc,container,object,pattern
def search_worker(item):
    search_object(*item)

skip_suffices=tuple(['.bam','.gz','.tif','.nc','.fcs','.dv','.mov','.bin',\
    '.jpg','.zip','.nd2','.lsm','.bz2','.avi','.pdf','.tgz','.xls','.png',\
    '.gif','.pyc','.ithmb','.wav','.pgm'])

def search_container(parse_arg):
    global skip_suffices

    # changed from obsolete psutil.phymem_usage().available
    memavail=psutil.virtual_memory().available

    sc=create_sw_conn(parse_arg.authtoken,parse_arg.storage_url)

    try:
        if parse_arg.prefix:
            headers,objs=sc.get_container(parse_arg.container,
                prefix=parse_arg.prefix,full_listing=True)
        else:
            headers,objs=sc.get_container(parse_arg.container,full_listing=True)

        search_pool=multiprocessing.Pool(parse_arg.maxproc)

        for obj in objs:
            if obj['name'].lower().endswith(skip_suffices) or\
                (parse_arg.filename and not \
                    fnmatch.fnmatch(obj['name'],parse_arg.filename)):
                    continue

            if obj['bytes']>memavail:
                print("Object",obj['name'],"too large for memory!",
                    file=sys.stderr)
                continue

            search_pool.apply_async(search_worker,[[parse_arg,obj['name']]])
            #search_object(parse_arg,obj['name'])

        search_pool.close()
        search_pool.join()

    except swiftclient.ClientException:
        print("Error: cannot access Swift container '%s'!" % 
            parse_arg.container)

    sc.close()

def parse_arguments():
    parser=argparse.ArgumentParser(
        description="Search text objects for pattern")
    parser.add_argument('-c','--container',required=True)
    parser.add_argument('pattern',type=str)
    parser.add_argument('-m','--maxproc',type=int,
        help="maximum number of processes to run",default=5)
    parser.add_argument('-a','--authtoken',
        default=os.environ.get("OS_AUTH_TOKEN"),
        help='swift authentication token (required when storage-url is used)')
    parser.add_argument('-s','--storage-url',
        default=os.environ.get("OS_STORAGE_URL"),
        help='swift storage url (required when authtoken is used)')
    parser.add_argument('-f','--filename',
        help='limit search to objects matching this pattern')
    parser.add_argument('-p','--prefix',
        help='limit search to objects matching this prefix')
    parser.add_argument('-b','--binary',action='store_true',
        help='try to exclude files identified as binary')
    parser.add_argument('-i','--insensitive',action='store_true',
        help='case insensitive search')

    return parser.parse_args()

def main():
    search_container(parse_arguments())

if __name__ == '__main__':
    main()
